import os
import sys
import math
import numpy
import pylab
import csv
import pyfits
import matplotlib.pyplot as plt
import matplotlib
from matplotlib.ticker import MaxNLocator


# runs a miriad command
# inputs:
#   list: a list of strings to be joined
#   log [optional]: an output log (not yet implemented)
def run(list, log=""):

    command = " ".join(list)

    # this way doesn't work, but it's here in case I want to mess
    # with it later.
    
    #lines = []
    #for line in os.popen(command, 'r', 1):
    #    lines.append(line)
    #    print line.split('\n')[0]
    #    print len(lines)
    #if os.path.exists(log):
    #    file = open(log, 'a')
    #    file.writelines(lines)
    #    file.close()

    # print the output to the screen and append to a file.
    if log:
        # uncomment this if you want to use tee instead of >>
        # make sure to comment the second command if you do so.
        
        #command = command + " | tee -a " + log
        command = command + " >> " + log
        
    os.system(command)

    return



# removes a group of files if they exist
def remove(files):
    for file in files:
        if os.path.exists(file):
            os.system("rm -r " + file)



# decides whether or not to run a reduction step
# inputs:
#   files: a list of files created by the task in question
#
# output:
#   boolean value: 1 if all the files exist and 0 if not all of them exist
def doStep(files):
    skipStep = bool(1)
    for file in files:
        skipStep = skipStep & os.path.exists(file)
    if not(skipStep):
        for file in files:
            remove(files)
            
    return not(skipStep)



# find the size of the beam given a *.beam file.
# input:
#   galaxy: base galaxy name
# output:
#   list of the form [bmaj, bmin] (fwhm of major axis and minor axis in arcsec)
def getdV(galaxy):
    list = ['itemize']
    list.append('in=' + galaxy + '.map')

    command = " ".join(list)

    # open a pipe to read the miriad output and grab the major axis
    pipe = os.popen(command + ' | grep cdelt3')
    output = pipe.read()
    output = output.split()

    dv = math.fabs(float(output[2]))
    
    
    return dv



def getBeamsize(galaxy):
    list = ['itemize']
    list.append('in=' + galaxy + '.beam')

    command = " ".join(list)

    # open a pipe to read the miriad output and grab the major axis
    pipe = os.popen(command + ' | grep bmaj')
    output = pipe.read()
    output = output.split()

    bmaj = float(output[2]) * 206265
    
    pipe = os.popen(command + " | grep bmin")
    output = pipe.read()
    output = output.split()
    
    bmin = float(output[2]) * 206265

    return [bmaj,bmin]


# finds the line free channels from the history
#
# you can have up to 2 pairs of line free channels (ie beginning and end)
#
def getLFChans(galaxy):

    # run prthis to find lf channel approximations from UVLSF inputs.
    list = ["prthis"]
    list.append("in=" + galaxy + ".map")
    list.append("log=hist.tmp.txt")
    list.append("task=uvlsf")

    run(list)

    pipe = os.popen("grep CHANSEL hist.tmp.txt")
    output = pipe.read()
    output = output.split("\n")
    
    lfchans = [0,0]
    # assumes that the last UVLSF entry is the one to use. Who knows.
    for line in output:
        tmp = line.split('=')
        if tmp[0] == "UVLSF CHANSEL( 1)":
            chans = tmp[1].split('/')[0].split(',')
            if len(lfchans):
                lfchans[0] = int(chans[0])
                lfchans[1] = int(chans[1])
            else:
                lfchans.append(int(chans[0]))
                lfchans.append(int(chans[1]))
                
        if tmp[0] == "UVLSF CHANSEL( 2)":
            chans = tmp[1].split('/')[0].split(',')

            if len(lfchans) == 2:
                lfchans.append(int(chans[0]))
                lfchans.append(int(chans[1]))
            elif len(lfchans) == 4:
                lfchans[2] = int(chans[0])
                lfchans[3] = int(chans[1])

        if tmp[0] == "UVLSF CHANSEL( 3)":
            chans = tmp[1].split('/')[0].split(',')
            if len(lfchans) == 4:
                lfchans.append(int(chans[0]))
                lfchans.append(int(chans[1]))
            elif len(lfchans) == 6:
                lfchans[4] = int(chans[0])
                lfchans[5] = int(chans[1])

        if tmp[0] == "UVLSF CHANSEL( 4)":
            chans = tmp[1].split('/')[0].split(',')
            if len(lfchans) == 6:
                lfchans.append(int(chans[0]))
                lfchans.append(int(chans[1]))
            elif len(lfchans) == 8:
                lfchans[6] = int(chans[0])
                lfchans[7] = int(chans[1])

    os.system("rm hist.tmp.txt")
    return lfchans

            

# determines the noise in a map.
def getNoise(galaxy, lfchans=[]):
    # units are Jy/beam, colums are:
    # 1: plane
    # 2: Felocity
    # 3: sum
    # 4: mean
    # 5: rms
    # 6: max
    # 7: min
    # 8: npoints
    
    
    # run imstat
    list = ["imstat"]
    list.append("in=" + galaxy +".map")
    list.append("options=guaranteespaces,noheader")
    list.append("log=imstat.tmp.txt")

    if lfchans:
        pass
    else:
        lfchans = getLFChans(galaxy)

    region = ""
    for i,chan in enumerate(lfchans):
        if i % 2:
            # it's odd
            region = region + str(chan) + "\),"
        else:
            region = region + "image\(" + str(chan) + ","

    # get rid of trailing comma
    region = region[0:len(region)-1]

    
#    if len(lfchans) == 2:
#        list.append("region=image\(" + str(lfchans[0]) + "," + \
#                    str(lfchans[1]) + "\)")
#    elif len(lfchans) == 4:
#        list.append("region=image\(" + str(lfchans[0]) + "," + \
#                    str(lfchans[1]) + "\)," + \
#                    "image\(" + str(lfchans[2]) + "," + str(lfchans[3]) + "\)")
    list.append("region="+region)
    run(list)
    
    # process imstat output
    rms = []
    chan = []
    reader = csv.reader(open("imstat.tmp.txt","rb"))
    reader_list = []
    reader_list.extend(reader)
    
    for line in reader_list:
        tmp = line[0]
        tmp = tmp.split()
        if tmp[0] == "All":
            pass
        else:
            rms.append(float(tmp[4]))
            chan.append(int(tmp[0]))

    
    pylab.plot(chan, rms)
    pylab.show()
    
    
    # find the median:
    rms.sort()
    n = len(rms)
    if n & 1:
        noise = rms[n // 2]
    else:
        noise = (rms[n // 2 - 1] + rms[n // 2]) / 2.0

    
    os.system("rm imstat.tmp.txt")

    # return the noise in mJy
    return noise*1000
    


# read in the map and beam fits files and run velsw on the map to switch
# to the correct velocity units
def readIn(galaxy):

    # write to the log:
    f = open(galaxy + ".image.log", mode="a")
    f.write("\n")
    f.write("image.py READIN:\n")
    f.close()
    
    print "Running fits on the map!"
    # run fits on the map
    list = ['fits']
    list.append('in=' + galaxy + '.isubim.1.fits')
    list.append('op=xyin')
    list.append('out=' + galaxy + '.map')

    run(list, log=galaxy + ".image.log")

    print "Running fits on the beam!"
    # run fits on the beam
    list = ['fits']
    list.append('in=' + galaxy + '.bsubim.1.fits')
    list.append('op=xyin')
    list.append('out=' + galaxy + '.beam')

    run(list, log=galaxy + ".image.log")

    # run velsw on the map
    print "Running velsw on the map!"
    list = ['velsw']
    list.append('in=' + galaxy + '.map')
    list.append('axis=opt,bary')

    run(list, log=galaxy + ".image.log")


# cleans to some depth with an optional mask. this task also restores
# the clean components/residuals/c+r map and corrects for the primary
# beam.
# Note: depth is in mJy
def clean(galaxy, depth, mask="", niters=0):

    # for clarity later.
    if mask:
        m = "m"
        # write to the log:
        f = open(galaxy + ".image.log", mode="a")
        f.write("\n")
        f.write("image.py RECLEAN:\n")
        f.close()

    else:
        m = ""
        # write to the log:
        f = open(galaxy + ".image.log", mode="a")
        f.write("\n")
        f.write("image.py CLEAN:\n")
        f.close()
        
    if m:
        print "Running masked clean!"
    else:
        print "Running initial clean!"
    # run clean
    list = ['clean']
    list.append('map=' + galaxy + '.map')
    list.append('beam=' + galaxy + '.beam')
    if niters:
        list.append("niters=" + str(niters))
    else:
        list.append("niters=50000")
    list.append('cutoff=' + str(depth * 0.001))
    list.append("out=" + galaxy + "." +  m + "clean")

    if mask:
        list.append('region=mask\(' + galaxy + '.mask\)')

    run(list, log=galaxy + ".image.log")

    print "Running restor for c+r map!"
    # run restor
    list = ["restor"]
    list.append("map=" + galaxy + ".map")
    list.append("beam=" + galaxy + ".beam")

    list.append("model=" + galaxy + "." + m + "clean")
    list.append("out=" + galaxy + "." + m + "restor")

    run(list, log=galaxy + ".image.log")

    # residual map
    list.append("mode=resid")
    list[4] = "out=" + galaxy + ".resid." + m + "restor"

    print "Running restor for residual map!"
    run(list, log=galaxy + ".image.log")

    print "Running maths to find cc map!"
    # subtract to get clean components...
    list=["maths"]
    list.append("exp=\<" + galaxy  + "." + m + \
                "restor\>-\<" + galaxy + ".resid." + m + "restor\>")
    list.append("out=" + galaxy + ".cc." + m + "restor")

    run(list)

    # correct for the primary beam with linmos
    list = ["linmos"]
    list.append("in=" + galaxy + "." + m + "restor")
    list.append("out=" + galaxy + "." + m + "restor.linmos")

    run(list)


# smooth finds the beam size, convolves the .restor map with
# twice the beam, hanning smooths.
def smooth(galaxy):
    
    # write to the log:
    f = open(galaxy + ".image.log", mode="a")
    f.write("\n")
    f.write("image.py SMOOTH:\n")
    f.close()

    # get the beam size
    bmaj = math.ceil(getBeamsize(galaxy)[0])

    print "Running convol!"
    # convol
    list = ['convol']
    list.append('map=' + galaxy + '.cc.restor')
    list.append('out=' + galaxy + '.cc.restor.convol')
    list.append('fwhm=' + str(2 * bmaj) + ',' + str(2 * bmaj))
    list.append('options=final')

    run(list, log=galaxy + ".image.log")

    print "Running hanning!"
    # hanning
    list = ['hanning']
    list.append('in=' + galaxy + '.cc.restor.convol')
    list.append('out=' + galaxy + '.cc.restor.convol.hanning')
    list.append('width=3')

    run(list, log=galaxy + ".image.log")

    os.system("rm -r " + galaxy + ".cc.restor.convol")
    
    

# mask runs cgcurs and then immask.    
def mask(galaxy, noise):

    # write to the log:
    f = open(galaxy + ".image.log", mode="a")
    f.write("\n")
    f.write("image.py MASK:\n")
    f.close()

    # blank
    list = ['maths']
    list.append('exp=\<' + galaxy + '.cc.restor.convol.hanning\>')
    list.append('mask=\<' + galaxy + '.cc.restor.convol.hanning\>.' +
                'gt.'+ str(noise*0.001))
    list.append('out=' + galaxy + '.mask')

    print "Blanking cube!"
    run(list, log=galaxy + ".image.log")

    # cgcurs
    list = ['cgcurs']
    list.append('in=' + galaxy + '.mask')
    list.append('type=pix')
    list.append('device=/xwin')
    list.append('range=0,0.00001')
    list.append('nxy=1')
    list.append("options=3pixel,3value,region")

    print "Running cgcurs!"
    run(list)
    os.system('mv cgcurs.region ' + galaxy + '.region')
    

    print "Creating the mask!"
    # mask
    list = ["immask"]
    list.append("in=" + galaxy + ".mask")
    list.append("region=@" + galaxy + ".region")
    list.append("logic=and")
    list.append("flag=true")

    run(list, log=galaxy + ".image.log")


def moments(galaxy, noise):

    # write to the log:
    f = open(galaxy + ".image.log", mode="a")
    f.write("\n")
    f.write("image.py MOMENTS:\n")
    f.close()


    # mask the restored cube - should already be done
    #list = ["immask"]
    #list.append("in=" + galaxy + ".mrestor.jvm")
    #list.append("region=mask\(" + galaxy + ".mask\)")
    #list.append("logic=and")
    #list.append("flag=true")
    #run(list)


    # run zeroth moment
    list = ["moment"]
    list.append("in=" + galaxy + ".mrestor.jvm")
    list.append("out=" + galaxy + ".m0")
    list.append("mom=0")
    
    run(list)

    # correct m0 for the primary beam with linmos - already done
    #list = ["linmos"]
    #list.append("in=" + galaxy + ".m0.tmp")
    #list.append("out=" + galaxy + ".m0")
    #run(list)
    
    # remove un-corrected m0
    #os.system("rm -r " + galaxy + ".m0.tmp")

    # first moment
    list = ["moment"]
    list.append("in=" + galaxy + ".mrestor.jvm")
    list.append("out=" + galaxy + ".m1.tmp")
    list.append("mom=1")
    run(list)

    # second moment
    list[2] = "out=" + galaxy + ".m2.tmp"
    list[3] = "mom=2"
    run(list)

    # -2 moment - this will be converted to kelvin
    list[2] = "out=" + galaxy + ".m-2.tmp"
    list[3] = "mom=-2"
    run(list)


    # convolve the m0 map with 2x the beam size
    beam = getBeamsize(galaxy)
    bmaj = beam[0]
    bmin = beam[1]


    # convol
    list = ['convol']
    list.append('map=' + galaxy + '.m0')
    list.append('out=' + galaxy + '.m0.convol')
    list.append('fwhm=' + str(2 * bmaj) + ',' + str(2 * bmaj))
    list.append('options=final')

    run(list)

    # convert to column density
    list = ["maths"]
    list.append('exp=\<' + galaxy + '.m0.convol\>*1.222e6/'+ \
                str(4 * bmaj * bmaj) + "/1.420/1.420*1.8224e18")
                # 4 * bmaj * bmaj is because we convolved with a larger beam.
    list.append('out=' + galaxy + '.m0.convol.cd')

    run(list)
    
    
    # get the units of the file right
    list = ["puthd"]
    list.append("in=" + galaxy + ".m0.convol.cd/bunit")
    list.append("value=cm^-2")

    run(list)
    


    # mask mom1 at 5e20 column density
    list = ["maths"]
    list.append("exp=\<" + galaxy  + ".m1.tmp\>")
    list.append("mask=\<" + galaxy  + ".m0.convol.cd\>.ge.5e20")
    list.append("out=" + galaxy + ".m1")

    run(list)

    # mask mom2 at 5e20 column density
    list = ["maths"]
    list.append("exp=\<" + galaxy  + ".m2.tmp\>")
    list.append("mask=\<" + galaxy  + ".m0.convol.cd\>.ge.5e20")
    list.append("out=" + galaxy + ".m2")

    run(list)
    
    # remove .tmp moment maps
    os.system("rm -r " + galaxy + ".m?.tmp")

    # remove m0.convol files
    os.system("rm -r " + galaxy + ".m0.convol*")
    
    # convert m-2 to kelvin units
    list = ["maths"]
    list.append("exp=\<" + galaxy + ".m-2.tmp\>*1.222e6/1.4/1.4/" + \
                str(bmaj) + "/" + str(bmin))
    list.append("out=" + galaxy + ".m-2")
    run(list)
    
    # put the proper units in the header
    list = ["puthd"]
    list.append("in=" + galaxy + ".m-2/bunit")
    list.append("value=K")
    
    run(list)
    
    os.system("rm -r " + galaxy + ".m-2.tmp")
    
    # delete the mask from the .mrestor file
    #os.system("rm " + galaxy + ".mrestor./mask")


# creates spectrum with jvm correction.
def jvm(galaxy):
    # eD = C + eR
    # e(D - R) = C
    # e = C / (D - R)
    # 
    # G = C*D / (D - R)
	
	# write to the log:
    f = open(galaxy + ".image.log", mode="a")
    f.write("\n")
    f.write("image.py JVM:\n")
    f.close()


    # remove linmos files
    if os.path.exists(galaxy + ".map.linmos"):
        os.system("rm -r " + galaxy + ".map.linmos")
    if os.path.exists(galaxy + ".resid.mrestor.linmos"):
        os.system("rm -r " + galaxy + ".resid.mrestor.linmos")
    if os.path.exists(galaxy + ".cc.mrestor.linmos"):
        os.system("rm -r " + galaxy + ".cc.mrestor.linmos")

        
    # correct all for the primary beam
    list = ["linmos"]
    list.append("in=" + galaxy + ".map")
    list.append("out=" + galaxy + ".map.linmos")
    run(list)
    
    list[1] = "in=" + galaxy + ".cc.mrestor"
    list[2] = "out=" + galaxy + ".cc.mrestor.linmos"
    run(list)
    
    list[1] = "in=" + galaxy + ".resid.mrestor"
    list[2] = "out=" + galaxy + ".resid.mrestor.linmos"
    run(list)
    
    
    # mask the resid, cc, dirty cubes.
    list = ["immask"]
    list.append("in=" + galaxy + ".resid.mrestor.linmos")
    list.append("region=mask\(" + galaxy + ".mask\)")
    list.append("logic=and")
    list.append("flag=true")

    run(list)
    
    list[1] = "in=" + galaxy + ".map.linmos"
    run(list)
    
    list[1] = "in=" + galaxy + ".cc.mrestor.linmos"
    run(list)
        
    
    # run these through imstat.
    list = ["imstat"]
    list.append("in=" + galaxy +".map.linmos")
    list.append("options=guaranteespaces,noheader")
    list.append("log=imstat.dirty.tmp.txt")
    run(list)
    
    list = ["imstat"]
    list.append("in=" + galaxy +".cc.mrestor.linmos")
    list.append("options=guaranteespaces,noheader")
    list.append("log=imstat.cc.tmp.txt")
    run(list)

    list = ["imstat"]
    list.append("in=" + galaxy +".resid.mrestor.linmos")
    list.append("options=guaranteespaces,noheader")
    list.append("log=imstat.resid.tmp.txt")
    run(list)
    
    
    # read in imstat output
    npix = []
    felocity = []
    flux = []
    epsilon = []
    c = []
    d = []
    r = []
    plane = []
    reader = csv.reader(open("imstat.resid.tmp.txt","rb"))
    reader_list = []
    reader_list.extend(reader)
    for line in reader_list:
        tmp = line[0]
        tmp = tmp.split()
        if tmp[0] == "All":
            pass
        else:
            r.append(float(tmp[3]))
            # npix and felocity should be the same for all
            # maps at this channel.
            npix.append(int(tmp[7]))
            felocity.append(float(tmp[1]))
            plane.append(float(tmp[0]))
            
            
    reader = csv.reader(open("imstat.cc.tmp.txt","rb"))
    reader_list = []
    reader_list.extend(reader)
    for line in reader_list:
        tmp = line[0]
        tmp = tmp.split()
        if tmp[0] == "All":
            pass
        else:
            c.append(float(tmp[3]))
	
    reader = csv.reader(open("imstat.dirty.tmp.txt","rb"))
    reader_list = []
    reader_list.extend(reader)
    for line in reader_list:
        tmp = line[0]
        tmp = tmp.split()
        if tmp[0] == "All":
            pass
        else:
            d.append(float(tmp[3]))

    # figure out constants
    beam = getBeamsize(galaxy)
    bmaj = beam[0]
    bmin = beam[1]
    
    npixPerBeam = 1.133 * bmaj * bmin / 1.5 / 1.5 # cellsize is 1.5"
    
    # get dv - channel width
    dv = getdV(galaxy)
    
    # loop through and figure out total flux and integrated flux
    fint = 0
    for i in range(len(d)):
        d[i] = d[i] * npix[i] / npixPerBeam
        c[i] = c[i] * npix[i] / npixPerBeam
        r[i] = r[i] * npix[i] / npixPerBeam
        if math.fabs(d[i] - r[i]) < 0.00001:
            epsilon.append(0)
            flux.append(0)
        else:
            epsilon.append(c[i] / (d[i] - r[i]))
            flux.append(epsilon[i] * d[i])
        fint = fint + flux[i] * dv
    
    pylab.plot(felocity, flux)
    pylab.show()
    
    # write these out to a file
    f = open(galaxy + ".spect.txt","w")
    f.write("# Columns: plane, felocity, flux, epsilon, npix, d, c, r\n")
    for i in range(len(d)):
        f.write(str(plane[i]) + "\t" + str(felocity[i]) + "\t" + \
                str(round(flux[i]*1000000)/1000000) + "\t" + \
                str(round(epsilon[i]*1000000)/1000000) + "\t" + \
                str(round(npix[i]*1000000)/1000000) + "\t" + \
                str(round(d[i]*1000000)/1000000) + "\t" + \
                str(round(c[i]*1000000)/1000000) + "\t" + \
                str(round(r[i]*1000000)/1000000) + "\n")
    f.write("# Total Integrated Flux, corrected: " + \
            str(math.ceil(fint*1000)/1000) + " Jy km/s\n")
    f.close()
    
    os.system("cp " + galaxy + ".spect.txt data.dat")
    os.system("$red/../SCRIPTS/jvm.sh data.dat")
    os.system("mv galaxy.ps " + galaxy + ".spect.ps")
    os.system("rm data.dat")
    
    # clean up imstat files
    os.system("rm imstat.*.tmp.txt")
    
    # write out files to fits and run idl jvm correction program
    list = ["fits"]
    list.append("in=" + galaxy + ".cc.mrestor.linmos")
    list.append("op=xyout")
    list.append("out=" + galaxy + ".tmp.cc.fits")
    run(list)

    list = ["fits"]
    list.append("in=" + galaxy + ".resid.mrestor.linmos")
    list.append("op=xyout")
    list.append("out=" + galaxy + ".tmp.resid.fits")
    run(list)
    
    #idl jvm call
    os.system("idl<<EOF\njvm,'" + galaxy + "'\nEOF")
    
    # I don't think we need this in Miriad format, but we can read it in
    # here with fits if we want to
    # uncomment if you want it.
    list = ["fits"]
    list.append("in=" + galaxy + ".mrestor.jvm.fits")
    list.append("op=xyin")
    list.append("out=" + galaxy + ".mrestor.jvm")
    run(list)


    # clean up temp fits files
    remove([galaxy + ".tmp.resid.fits",galaxy + ".tmp.cc.fits"])
    
    # remove linmos files
    os.system("rm -r " + galaxy + ".map.linmos")
    os.system("rm -r " + galaxy + ".resid.mrestor.linmos")
    os.system("rm -r " + galaxy + ".cc.mrestor.linmos")
    

def coldens(galaxy):

    # convolve the m0 map with 2x the beam size
    beam = getBeamsize(galaxy)
    bmaj = beam[0]
    bmin = beam[1]


    # convol - we are not running it here, we just want column density
    #list = ['convol']
    #list.append('map=' + galaxy + '.m0')
    #list.append('out=' + galaxy + '.m0.convol')
    #list.append('fwhm=' + str(2 * bmaj) + ',' + str(2 * bmaj))
    #list.append('options=final')

    #run(list)

    # convert to column density
    list = ["maths"]
    list.append('exp=\<' + galaxy + '.m0\>*1.222e6/'+ \
                str(bmaj * bmin) + "/1.420/1.420*1.8224e18")
    list.append('out=' + galaxy + '.m0.cd')

    run(list)
    
    
    # get the units of the file right
    list = ["puthd"]
    list.append("in=" + galaxy + ".m0.cd/bunit")
    list.append("value=cm^-2")

    run(list)


def spect(galaxy):
    # get rid of pyspect files if they exist
    remove([galaxy + ".pyspect.txt", galaxy + ".pyspect.ps"])
    
    # make sure the .jvm.fits file exists
    if not os.path.exists(galaxy + ".mrestor.jvm.fits"):
        if os.path.exists(galaxy + ".mrestor.jvm"):
            list = ["fits"]
            list.append("in=" + galaxy + ".mrestor.jvm")
            list.append("out=" + galaxy + ".mrestor.jvm.fits")
            list.append("op=xyout")

            run(list)

    # read it in to python
    cube = pyfits.getdata(galaxy + ".mrestor.jvm.fits")
    header = pyfits.getheader(galaxy + ".mrestor.jvm.fits")

    # deal with nans - just set them to 0.
    cube[numpy.isnan(cube).nonzero()] = 0

    # calculate the spectra - don't need npix and the mean, just the sum:
    # <flux> * npix * beam / pix = sum * beam / pix
    bmaj, bmin = getBeamsize(galaxy)
    spect = numpy.sum(numpy.sum(cube, axis=1), axis=1)
    spect = spect * (1.5**2) / (1.133 * bmaj * bmin)

    # get the velocity axis
    velocity = numpy.arange(header['naxis3'])
    slope = header['cdelt3']
    intercept = header['crval3'] - slope * (header['crpix3']-1)
    velocity = slope * velocity + intercept
    velocity = velocity / 1.0e3

    ind = (spect > 0).nonzero()

    #matplotlib.rc("xtick", labelsize=12)
    #matplotlib.rc("ytick", labelsize=12)
    matplotlib.rc('font', family="serif")
    matplotlib.rc('font', size=24)
    matplotlib.rc('axes', linewidth=2)
    matplotlib.rc("xtick.major", size = 10)
    matplotlib.rc("xtick.major", pad = 10)
    matplotlib.rc("ytick.major", size = 12)
    matplotlib.rc("ytick.major", pad = 10)


    # now plot it
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.plot(velocity[ind], spect[ind], 'k', linewidth=2)
    ax.xaxis.set_major_locator(MaxNLocator(5))
    ax.yaxis.set_major_locator(MaxNLocator(5, steps=[0.1, 0.2, 0.5, 1, 2, 5, 10]))
    plt.text(0.9, 0.85, ((galaxy.partition("-"))[0]).upper(), \
             transform=ax.transAxes, horizontalalignment="right")
    #plt.title(((galaxy.partition("-"))[0]).upper())
    plt.subplots_adjust(bottom=0.15, left=0.17)

    ax2 = plt.gca()
    for line in ax.xaxis.get_ticklines():
        line.set_markeredgewidth(2)

    for line in ax.yaxis.get_ticklines():
        line.set_markeredgewidth(2)


    plt.xlabel("Velocity (km/s)")
    plt.ylabel("Flux Density (Jy)")

    plt.savefig(galaxy + ".pyspect.eps")
    # write the spectrum out to a file
    file = open(galaxy + ".pyspect.txt", 'w')
    file.write("# channel \t velocity \t fluxdensity\n")
    file.write("#         \t (km/s) \t (Jy)\n")
    for chan, vel, flux in zip(numpy.arange((spect.shape)[0]), \
                               velocity,
                               spect):
        file.write(str(chan) + "\t" + \
                   str(round(vel * 100) / 100.0) + "\t" + \
                   str(flux) + "\n")

    file.close()
    
    


# deletes unnecessary miriad files, writes others out to fits, and makes a tarball.
def cleanup(galaxy, createdFiles, paramFile):

    # check that tar file doesn't exist.
    if os.path.exists(galaxy + ".image.tar"):
        print "File " + galaxy + ".image.tar exists. Please delete " + \
              "before continuing."
        return
    
    #write out "keep" files to fits
    for file in createdFiles["keep"]:
        if file == galaxy + ".spect.ps" or file == galaxy + ".spect.txt" or \
           file == galaxy + ".image.log":
            pass
        else:
            if os.path.exists(file + ".fits"):
                os.system("rm " + file + ".fits")
            list = ["fits"]
            list.append("in=" + file)
            list.append("out=" + file + ".fits")
            list.append("op=xyout")
        
            run(list)
    
    # tar up the fits files
    command = "tar --create --file=" + galaxy + ".image.tar " + paramFile
    for file in createdFiles["keep"]:
        if file == galaxy + ".spect.ps" or file == galaxy + ".spect.txt" or \
           file == galaxy + ".image.log":
            command = command + " " + file
        else:
            command = command + " " + file + ".fits"

    command = command + " " + galaxy + ".isubim.1.fits " + galaxy + \
              ".bsubim.1.fits"
    os.system(command)
    
    # remove all the miriad files (including the "keep" ones because we have
    # them in fits format)
    remove(createdFiles["readin"])
    remove(createdFiles["clean"])
    remove(createdFiles["smooth"])
    remove(createdFiles["mask"])
    remove(createdFiles["reclean"])
    remove(createdFiles["moments"])



def recover(galaxy, keepFiles):
    if os.path.exists(galaxy + ".image.tar"):
        os.system("tar -xvf " + galaxy + ".image.tar")
    else:
        print "File " + galaxy + ".image.tar does not exist." 
        return

    if os.path.exists(galaxy + ".isubim.1.fits") and \
       os.path.exists(galaxy + ".bsubim.1.fits"):
        readin(galaxy)
    else:
        print "Files " + galaxy + ".isubim.1.fits and " + galaxy + \
              ".bsubim.1.fits are needed to recover setup."
        return
    
    for file in keepFiles:
        if file == galaxy + ".spect.ps" or file == galaxy + ".spect.txt" \
            or file == galaxy + ".image.log":
            pass
        else:
            list = ["fits"]
            list.append("in=" + file + ".fits")
            list.append("out=" + file)
            list.append("op=xyin")

            run(list)

	
# sets up the log.
# this includes erasing lines for steps that will be repeated.
def resetLog(galaxy, restart=""):

    if os.path.exists(galaxy + ".image.log"):
        f = open(galaxy + ".image.log", mode='r')
        lines = f.readlines()
        f.close()

    else:
        return -1

    if restart:
        i = 0
        if restart == "readin":
            # start over.
            i = 0

        
        elif restart == "clean":

            try:
                i = lines.index("image.py CLEAN:\n")
            except ValueError:
                # it's not found.
                i=0
                print '"image.py CLEAN" not found in log file.'
                return
                

        elif restart == "smooth":
            try:
                i = lines.index("image.py SMOOTH:\n")
            except ValueError:
                # it's not found.
                i=0
                print '"image.py SMOOTH" not found in log file.'
                return


        elif restart == "mask":
            try:
                i = lines.index("image.py MASK:\n")
            except ValueError:
                # it's not found.
                i=0
                print '"image.py MASK" not found in log file.'
                return


        elif restart == "reclean":
            try:
                i = lines.index("image.py RECLEAN:\n")
            except ValueError:
                # it's not found.
                i = 0
                print '"image.py RECLEAN" not found in log file.'
                return

        elif restart == "moments":
            try:
                i = lines.index("image.py MOMENTS:\n")
            except ValueError:
                # it's not found
                i = 0
                print '"image.py MOMENTS" not found in log file.'
                return
                
        elif restart == "jvm":
            try:
                i = lines.index("image.py JVM:\n")
            except ValueError:
                # it's not found
                i = 0
                print '"image.py JVM" not found in log file.'
                return


        # remove the old file and write the new one.
        os.system("rm " + galaxy + ".image.log")
        if i:
            f = open(galaxy + ".image.log", "w")
            f.writelines(lines[:i])
            f.close()

